"""App URLs"""

# Django
from django.urls import path

# AA Mumble Quick Connect
from aa_mumble_quick_connect import views

app_name: str = "aa_mumble_quick_connect"

urlpatterns = [
    path(route="", view=views.index, name="index"),
    path(route="<slug:section>/", view=views.index, name="index"),
]



"""
Admin models
"""

# Django
from django.contrib import admin

# AA Mumble Quick Connect
from aa_mumble_quick_connect.models import MumbleLink, Section


@admin.register(Section)
class SectionAdmin(admin.ModelAdmin):
    """
    SectionAdmin
    """

    list_display = ("name",)
    ordering = ("name",)


@admin.register(MumbleLink)
class MumbleLinkAdmin(admin.ModelAdmin):
    """
    MumbleLinkAdmin
    """

    list_display = ("name", "section", "url", "disable_url_verification")
    ordering = ("section", "name")



"""
Template tags for the aa-mumble-quick-connect app.
"""

# Django
from django import template

register = template.Library()


@register.simple_tag
def aa_mumble_quick_connect_link(channel_url=None, username=None):
    """
    Create a mumble quick connect link

    :param channel_url: Mumble channel URL
    :type channel_url: str
    :param username: Username
    :type username: str
    :return: Mumble quick connect link
    :rtype: str
    """

    return channel_url.replace("mumble://", f"mumble://{username}@")



"""
App Views
"""

# Django
from django.contrib.auth.decorators import login_required, permission_required
from django.core.handlers.wsgi import WSGIRequest
from django.http import HttpResponse
from django.shortcuts import render

# AA Mumble Quick Connect
from aa_mumble_quick_connect.dependency_checks import mumble_service_installed
from aa_mumble_quick_connect.models import MumbleLink, Section


@login_required
@permission_required("aa_mumble_quick_connect.basic_access")
@permission_required("mumble.access_mumble")
def index(request: WSGIRequest, section: str = None) -> HttpResponse:
    """
    Index view

    :param request: The request
    :type request: WSGIRequest
    :param section: The section (optional)
    :type section: str
    :return: The response
    :rtype: HttpResponse
    """

    channels_in_sections = Section.objects.prefetch_related("mumble_links").all()
    channels_without_sections = MumbleLink.objects.filter(section__isnull=True)

    context = {
        "mumble_service_installed": mumble_service_installed(),
        "channels_in_sections": channels_in_sections,
        "channels_without_sections": channels_without_sections,
        "section_highlight": section,
    }

    return render(
        request=request,
        template_name="aa_mumble_quick_connect/index.html",
        context=context,
    )



"""
Test the apps' template tags
"""

# Django
from django.template import Context, Template
from django.test import TestCase, override_settings


class TestQuickConnectLinkTemplateTag(TestCase):
    """
    Test aa_mumble_quick_connect template tag
    """

    @override_settings(DEBUG=False)
    def test_aa_mumble_quick_connect(self) -> None:
        """
        Test aa_mumble_quick_connect template tag

        :return:
        :rtype:
        """

        context = Context({"mumble_link": "mumble://example.com", "username": "foobar"})
        template_to_render = Template(
            template_string=(
                "{% load aa_mumble_quick_connect %}"
                "{% aa_mumble_quick_connect_link mumble_link username %}"
            )
        )

        rendered_template = template_to_render.render(context=context)
        expected = "mumble://foobar@example.com"

        self.assertEqual(first=rendered_template, second=expected)



"""
App Models
"""

# Django
from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext_lazy as _


def validate_mumble_url(url: str, disable_url_verification: bool = False) -> str:
    """
    Validate Mumble channel URL against the configured Mumble server

    :param url: URL to validate
    :type url: str
    :param disable_url_verification: Disable URL verification
    :type disable_url_verification: bool
    :return: Validated URL
    :rtype: str
    """

    if not url.startswith("mumble://"):
        raise ValidationError(
            {"url": _('The Mumble channel URL must start with "mumble://"')}
        )

    if not disable_url_verification:
        mumble_base_url = f"mumble://{settings.MUMBLE_URL}"

        if not url.startswith(mumble_base_url):
            raise ValidationError(
                {
                    "url": _(
                        _(f'The Mumble channel URL must start with "{mumble_base_url}"')
                    )
                }
            )

    return url


class General(models.Model):
    """
    Meta model for app permissions
    """

    class Meta:
        """
        Meta definitions
        """

        managed = False
        default_permissions = ()
        permissions = (("basic_access", _("Can access this app")),)


class Section(models.Model):
    """
    Section model
    """

    name = models.CharField(
        max_length=255, unique=True, help_text=_("Name of the section")
    )

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta definitions
        """

        default_permissions = ()
        verbose_name = _("Section")
        verbose_name_plural = _("Sections")

    def __str__(self) -> str:
        """
        String representation

        :return: Name of the section
        :rtype: str
        """

        return self.name


class MumbleLink(models.Model):
    """
    Mumble link model
    """

    section = models.ForeignKey(
        to=Section,
        on_delete=models.SET_NULL,
        related_name="mumble_links",
        verbose_name=_("Section"),
        null=True,
        blank=True,
        help_text=_("Section the Mumble channel belongs to. (Optional)"),
    )
    name = models.CharField(max_length=255, help_text=_("Name of the Mumble channel"))
    url = models.CharField(
        max_length=255,
        help_text=_("URL to the channel"),
    )
    disable_url_verification = models.BooleanField(
        default=False,
        verbose_name=_("Disable Mumble channel URL verification"),
        help_text=_(
            "If checked, the Mumble channel URL will not be verified against what is "
            "configured for this Auth instance when saving the link. Only use this "
            "if you are sure the URL is correct and the Mumble server is controlled "
            "by this Auth instance."
        ),
    )

    class Meta:  # pylint: disable=too-few-public-methods
        """
        Meta definitions
        """

        default_permissions = ()
        verbose_name = _("Mumble Link")
        verbose_name_plural = _("Mumble Links")

    def __str__(self) -> str:
        """
        String representation

        :return: Name of the Mumble channel
        :rtype: str
        """

        return self.name

    def clean(self):
        """
        Clean method

        :param args: Arguments
        :type args: tuple
        :param kwargs: Keyword arguments
        :type kwargs: dict
        :return: None
        :rtype: None
        """

        # Validate Mumble URL
        self.url = validate_mumble_url(
            url=self.url, disable_url_verification=self.disable_url_verification
        )

        super().clean()  # pragma: no cover



"""
Dependency checks for the `aa-mumble-quick-connect` app
"""

# Django
from django.apps import apps


def mumble_service_installed() -> bool:
    """
    Check if `allianceauth.services.modules.mumble` is installed and active

    :return: True if installed, False otherwise
    :rtype: bool
    """

    return apps.is_installed(app_name="allianceauth.services.modules.mumble")



# Generated by Django 4.2.19 on 2025-02-14 07:35

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aa_mumble_quick_connect", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="mumblelink",
            name="disable_url_verification",
            field=models.BooleanField(
                default=False,
                help_text="If checked, the Mumble channel URL will not be verified against what is configured for this Auth instance when saving the link. Only use this if you are sure the URL is correct and the Mumble server is controlled by this Auth instance.",
                verbose_name="Disable Mumble channel URL verification",
            ),
        ),
        migrations.AlterField(
            model_name="mumblelink",
            name="url",
            field=models.CharField(help_text="URL to the channel", max_length=255),
        ),
    ]
